import hypermedia.video.*;
import java.awt.Rectangle;
import processing.video.*;
import processing.serial.*;

Serial port;

OpenCV opencv;

int x, y, w, h, centerX, centerY, diffX, diffY;

Capture myCapture; 
String[] captureDevices;

int[] angles = new int[2];

float filtro = 0.25;

void setup() {

  //serial initialization 
  //println(Serial.list());
  port = new Serial(this, Serial.list()[0], 9600);
  
  // servo positions
  angles[0] = 90;
  angles[1] = 135;


  // image and capture initialization
  size( 320, 240 );

  String[] devices = Capture.list();
  println(devices);

  myCapture = new Capture(this, width, height, devices[3]);
  // myCapture.settings(); 

  opencv = new OpenCV( this );
  opencv.capture( width, height);                   // open video stream
  opencv.cascade( OpenCV.CASCADE_FRONTALFACE_ALT );  
  // load detection description, here-> front face detection : "haarcascade_frontalface_alt.xml"

}


public void stop() {
  opencv.stop();
  super.stop();
}

int Trunk(int value, int maxVal, int minVal)
{
  if (value > maxVal)
    return maxVal;
  else if (value <minVal)
    return minVal;
  else
    return value; 
}

void draw() {

  if (myCapture.available()) {

    myCapture.read();   
    opencv.copy(myCapture);
    opencv.flip( OpenCV.FLIP_VERTICAL );

    background(192);

    Rectangle[] faces= opencv.detect();

    // display the image
    image( opencv.image(), 0, 0 );
    
    // screen guides
    stroke(0, 100);
    line (0, height/2, width, height/2);
    line (width/2, 0, width/2, height); 

    if (faces.length > 0)
    {
      
      // interpolator
      
      int valx = (faces[0].x+faces[0].width/2)-width/2;
      valx=(int)((float)valx* 75.0/(float)width);
      int resultx = (int)(((float)(angles[0])*(1.0-filtro))+((float)(angles[0]+valx)*filtro));
      angles[0]= Trunk(resultx, 180, 0);

      int valy = (faces[0].y+faces[0].height/2)-height/2;
      valy=((int)((float)valy* 56.0/(float)height))/2;//*-1;
      int resulty = (int)(((float)(angles[1])*(1.0-filtro))+((float)(angles[1]+valy)*filtro));
      angles[1]= Trunk(resulty, 180, 0);
      
      
      // draw face area
      noFill();
      stroke(255,0,0, 200);
      line (x+w/2, y+h/2-10, x+w/2, y+h/2+10);
      line (x+w/2-10, y+h/2, x+w/2+10, y+h/2);
      
      stroke(255,0,0);
      for( int i=0; i<faces.length; i++ ) 
      {
        rect( faces[i].x, faces[i].y, faces[i].width, faces[i].height ); 
      }
    }
    
    else {
    
      int resultx = (int)(((float)(angles[0])*(1.0-0.1))+((float)(90)*0.1));
      angles[0]= Trunk(resultx, 180, 0);
      
      int resulty = (int)(((float)(angles[1])*(1.0-0.1))+((float)(135)*0.1));
      angles[1]= Trunk(resulty, 180, 0);
      
    }
    
      // send values to serial port - arduino
      port.write('A');
      port.write(angles[0]);
      port.write('B');
      port.write(angles[1]);
  }
}


